; Sets emacs variables based on mode.
; A list of (major-mode . ((var1 . value1) (var2 . value2)))
; Mode can be nil, which gives default values.

((nil . ((indent-tabs-mode . nil)
         (c-basic-offset . 4)))
)
